package com.pcloud.book.copyright.tools;

import com.pcloud.book.BookApplication;
import com.pcloud.book.book.constant.BookConstant;
import com.pcloud.book.copyright.dto.DateDTO;
import com.pcloud.channelcenter.base.constants.ChannelConstants;
import com.pcloud.common.entity.UploadResultInfo;
import com.pcloud.common.enums.ImageTypeEnum;
import com.pcloud.common.exceptions.BizException;
import com.pcloud.common.exceptions.FileException;
import com.pcloud.common.utils.DateUtils;
import com.pcloud.common.utils.UUIDUitl;
import com.pcloud.common.utils.aliyun.OssUtils;
import org.jbarcode.JBarcode;
import org.jbarcode.encode.Code128Encoder;
import org.jbarcode.paint.BaseLineTextPainter;
import org.jbarcode.paint.EAN13TextPainter;
import org.jbarcode.paint.WidthCodedPainter;
import org.springframework.boot.SpringApplication;
import org.springframework.stereotype.Component;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

/**
 * @author lily
 * @date 2018/12/4 16:37
 */
@Component("copyrightTools")
public class CopyrightTools {

   private static String[] chars = new String[]{"a", "b", "c", "d", "e", "f",
            "g", "h", "i", "j", "k", "m", "n", "o", "p", "q", "r", "s",
            "t", "u", "v", "w", "x", "y", "z", "0", "1", "2", "3", "4", "5",
            "6", "7", "8", "9", "A", "B", "C", "D", "E", "F", "G", "H",
            "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V",
            "W", "X", "Y", "Z"};


    public static String generateShortUuid() {
        StringBuffer shortBuffer = new StringBuffer();
        String uuid = UUID.randomUUID().toString().replace("-", "");
        for (int i = 0; i < 11; i++) {
            String str = uuid.substring(i * 2, i * 2 + 2);
            int x = Integer.parseInt(str, 16);
            shortBuffer.append(chars[x % 0x3C]);
        }
        return shortBuffer.toString().toLowerCase();
    }

   public static void main(String[] args) {
        long time = System.currentTimeMillis();
        System.out.println(createShortUuidByCount(100000));
        long time1 = System.currentTimeMillis();
        System.out.println(time1-time);
    }
    /**
     * 生成count位非重复的数
     *
     * @return
     */
    public static List<String> createShortUuidByCount(Integer count) {
        Map<String, String> numbers = new HashMap<>();
        Set<String> numSet = numbers.keySet();
        while (numSet.size() < count) {
            String uuid = generateShortUuid();
            numbers.put(uuid, uuid);
        }
        return new ArrayList<>(numbers.keySet());
    }


    public static String generateBarCode128Local(String strBarCode, String dimension, String barheight) {
        try {
            BufferedImage bi = null;
            JBarcode productBarcode = new JBarcode(Code128Encoder.getInstance(), WidthCodedPainter.getInstance(), EAN13TextPainter.getInstance());
            productBarcode.setXDimension(Double.valueOf(dimension));
            productBarcode.setBarHeight(Double.valueOf(barheight));
            productBarcode.setWideRatio(Double.valueOf(30.0D));
            productBarcode.setShowText(true);
            productBarcode.setTextPainter(BaseLineTextPainter.getInstance());
            bi = productBarcode.createBarcode(strBarCode);
            String imageUrl = uploadImage(bi);
//            File file = new File("BARCODE" + strBarCode + ".jpg");
//            ImageIO.write(bi, ImageTypeEnum.JPG.value, file);
            return imageUrl;
        } catch (Exception var7) {
            var7.printStackTrace();
            return "encodeError";
        }
    }


    public static String uploadImage(BufferedImage image) throws BizException {
        UploadResultInfo uploadResultInfo = null;
        try {
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            ImageIO.write(image, ImageTypeEnum.JPG.value, os);
            uploadResultInfo = OssUtils.uploadFileByte(os.toByteArray(), UUIDUitl.taskName(), ImageTypeEnum.JPG.value);
            os.close();
            return uploadResultInfo == null ? null : uploadResultInfo.getUrl();
        } catch (Exception e) {
            throw new FileException(FileException.FILE_UPLOAD_FAILURE, "上传条形码失败");
        }
    }

    /**
     * 获取开始与结束时间
     */
    public static DateDTO getDateByType(Long days) {
        DateDTO dateDTO = new DateDTO();
        if (BookConstant.SEVEN.equals(days)) {
            dateDTO.setStartTime(DateUtils.getLongDateStr(DateUtils.getDayStartTime(DateUtils.addDay(new Date(), -7))));
            dateDTO.setEndTime(DateUtils.getLongDateStr(DateUtils.getDayEndTime(DateUtils.addDay(new Date(), -1))));
        } else if (BookConstant.FIFTEEN.equals(days)) {
            dateDTO.setStartTime(DateUtils.getLongDateStr(DateUtils.getDayStartTime(DateUtils.addDay(new Date(), -15))));
            dateDTO.setEndTime(DateUtils.getLongDateStr(DateUtils.getDayEndTime(DateUtils.addDay(new Date(), -1))));
        } else if (BookConstant.THIRTY.equals(days)) {
            dateDTO.setStartTime(DateUtils.getLongDateStr(DateUtils.getDayStartTime(DateUtils.addDay(new Date(), -29))));
            dateDTO.setEndTime(DateUtils.getLongDateStr(DateUtils.getDayEndTime(new Date())));
        }
        return dateDTO;
    }

    public static List<String> getLast6Months() {
        DecimalFormat df = new DecimalFormat("00");
        List<String> last6Months = new ArrayList<>(6);
        Calendar cal = Calendar.getInstance();
        for (int i = 1; i < 7; i++) {
            last6Months.add(cal.get(Calendar.YEAR) + "-" + df.format(cal.get(Calendar.MONTH)+1)) ;
            // 逐次往前推1个月
            cal.set(Calendar.MONTH, cal.get(Calendar.MONTH) - 1);
        }
        return last6Months;
    }

}
